/** @file functions.i @brief luabindings for core generator functions */


/* FAU Discrete Event Systems Library (libfaudes)

   Copyright (C) 2008  Thomas Moor
   Exclusive copyright is granted to Klaus Schmidt

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with this library; if not, write to the Free Software
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA */


/*
**************************************************
**************************************************
**************************************************

Functions misc

**************************************************
**************************************************
**************************************************
*/


// **** Version
// Have function interface for a generator member
%{
void LuaVersion(const vGenerator& rGen, const std::string& ver, vGenerator& rRes)        { rGen.Version(ver,rRes);}
%}
// Tell SWIG with nice target name
%rename(Version) LuaVersion;
void LuaVersion(const vGenerator& rGen, const std::string& ver, vGenerator& rRes);

// **** Determinism
// Have function interface for a generator member
%{
bool LuaIsDeterministic(const vGenerator& rGen)        { return rGen.IsDeterministic();}
%}
// Tell SWIG with nice target names
void Deterministic(const vGenerator& rGen, vGenerator& rResGen);
%rename(IsDeterministic) LuaIsDeterministic;
bool LuaIsDeterministic(const vGenerator& rGen);


// *** State minimisation
// Tell SWIG
void StateMin(const vGenerator& rGen, vGenerator& rResGen);

// *** Projection
// Tell SWIG with nice target names
%rename(ProjectNonDet) cProjectNonDet;
void cProjectNonDet(cGenerator& rGen, const EventSet& rAlph);
%rename(Project) cProject;
void cProject(const cGenerator& rGen, const EventSet& rAlph, cGenerator& rResGen);
%rename(InvProject) cInvProject;
void cInvProject(cGenerator& rGen, const cEventSet& rAlph);


/*
**************************************************
**************************************************
**************************************************

Functions from regex and friends

**************************************************
**************************************************
**************************************************
*/

// Tell SWIG
void LanguageConcatenateNonDet(const vGenerator& rGen1, const vGenerator& rGen2, vGenerator& rRes);
void LanguageIntersection(const vGenerator& rGen1, const vGenerator& rGen2, vGenerator& rRes);
void LanguageUnion(const vGenerator& rGen1, const vGenerator& rGen2, vGenerator& rRes);
void LanguageUnionNonDet(const vGenerator& rGen1, const vGenerator& rGen2, vGenerator& rResGen);
void KleeneClosureNonDet(vGenerator& rGen);


/*
**************************************************
**************************************************
**************************************************

Functions related to EventSet

**************************************************
**************************************************
**************************************************
*/

// Have function interface with cEventSets
%{
void LuaEventSetUnion(const EventSet& rAlph1, const EventSet& rAlph2, EventSet& rRes) 
 { rRes = rAlph1; rRes.InsertSet(rAlph2); }
void LuaEventSetIntersection(const EventSet& rAlph1, const EventSet& rAlph2, EventSet& rRes)
 { rRes = rAlph1 * rAlph2; }
void LuaEventSetDifference(const EventSet& rAlph1, const EventSet& rAlph2, EventSet& rRes) 
 { rRes = rAlph1; rRes.EraseSet(rAlph2); }
%}

// Tell SWIG with nice target names
%rename(EventSetUnion) LuaEventSetUnion;
%rename(EventSetIntersection) LuaEventSetIntersection;
%rename(EventSetDifference) LuaEventSetDifference;
void LuaEventSetUnion(const EventSet& rAlph1, const EventSet& rAlph2, EventSet& rRes);
void LuaEventSetIntersection(const EventSet& rAlph1, const EventSet& rAlph2, EventSet& rRes);
void LuaEventSetDifference(const EventSet& rAlph1, const EventSet& rAlph2, EventSet& rRes);


/*
**************************************************
**************************************************
**************************************************

Help topics

**************************************************
**************************************************
**************************************************
*/



SwigHelpTopic("Functions","Core library functions");

SwigHelpEntry("Functions","regular expressions", " LanguageConcatenateNonDet(gen1_arg, gen2_arg, gen_res)");
SwigHelpEntry("Functions","regular expressions", " LanguageUnion(gen1_arg, gen2_arg, gen_res)");
SwigHelpEntry("Functions","regular expressions", " LanguageUnionNonDet(gen1_arg, gen2_arg, gen_res)");
SwigHelpEntry("Functions","regular expressions", " LanguageIntersection(gen1_arg, gen2_arg, gen_res)");
SwigHelpEntry("Functions","regular expressions", " KleeneClosureNonDet(gen)");


SwigHelpEntry("Functions","event sets"," EventSetUnion(alph1_arg, alph2_arg, alph_res)");
SwigHelpEntry("Functions","event sets"," EventSetIntersection(alph1_arg, alph2_arg, alph_res)");
SwigHelpEntry("Functions","event sets"," EventSetDifference(alph1_arg, alph2_arg, alph_res)");

SwigHelpEntry("Functions","generator misc"," Version(gen_arg, string_arg, gen_res)");
SwigHelpEntry("Functions","generator misc","bool IsDeterministic(gen_arg)");
SwigHelpEntry("Functions","generator misc"," Deterministic(gen_arg, gen_res)");
SwigHelpEntry("Functions","generator misc"," StateMin(gen_arg, gen_res)");
SwigHelpEntry("Functions","generator misc"," ProjectNonDet(gen, alph)");
SwigHelpEntry("Functions","generator misc"," Project(gen_arg, alph, gen_res)");
SwigHelpEntry("Functions","generator misc"," InvProject(gen, alph)");



